'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.presets = exports.compose = exports.$ = exports.plugins = exports.styleSheet = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.simulations = simulations;
exports.simulate = simulate;
exports.cssLabels = cssLabels;
exports.flush = flush;
exports.insertRule = insertRule;
exports.idFor = idFor;
exports.isRule = isRule;
exports.add = add;
exports.style = style;
exports.active = active;
exports.any = any;
exports.checked = checked;
exports.disabled = disabled;
exports.empty = empty;
exports.enabled = enabled;
exports._default = _default;
exports.first = first;
exports.firstChild = firstChild;
exports.firstOfType = firstOfType;
exports.fullscreen = fullscreen;
exports.focus = focus;
exports.hover = hover;
exports.indeterminate = indeterminate;
exports.inRange = inRange;
exports.invalid = invalid;
exports.lastChild = lastChild;
exports.lastOfType = lastOfType;
exports.left = left;
exports.link = link;
exports.onlyChild = onlyChild;
exports.onlyOfType = onlyOfType;
exports.optional = optional;
exports.outOfRange = outOfRange;
exports.readOnly = readOnly;
exports.readWrite = readWrite;
exports.required = required;
exports.right = right;
exports.root = root;
exports.scope = scope;
exports.target = target;
exports.valid = valid;
exports.visited = visited;
exports.dir = dir;
exports.lang = lang;
exports.not = not;
exports.nthChild = nthChild;
exports.nthLastChild = nthLastChild;
exports.nthLastOfType = nthLastOfType;
exports.nthOfType = nthOfType;
exports.after = after;
exports.before = before;
exports.firstLetter = firstLetter;
exports.firstLine = firstLine;
exports.selection = selection;
exports.backdrop = backdrop;
exports.placeholder = placeholder;
exports.select = select;
exports.parent = parent;
exports.merge = merge;
exports.media = media;
exports.trackMediaQueryLabels = trackMediaQueryLabels;
exports.fontFace = fontFace;
exports.keyframes = keyframes;
exports.cssFor = cssFor;
exports.attribsFor = attribsFor;
exports.log = log;

var _sheet = require('./sheet.js');

var _plugins = require('./plugins');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

var _CSSPropertyOperations = require('react/lib/CSSPropertyOperations');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } /**** stylesheet  ****/


// we include these by default 

// these here are our main 'mutable' references
var styleSheet = exports.styleSheet = new _sheet.StyleSheet({ name: '_css_' });
// an isomorphic StyleSheet shim. hides all the nitty gritty. 

styleSheet.cache = {}; // hang on some state on to this instance 
styleSheet.inserted = {}; // ids of rules inserted 

// plugins 
var plugins = exports.plugins = styleSheet.plugins = new _plugins.PluginSet(_plugins.fallbacks, _plugins.prefixes);
plugins.media = new _plugins.PluginSet(); // neat! media, font-face, keyframes
plugins.fontFace = new _plugins.PluginSet();
plugins.keyframes = new _plugins.PluginSet(_plugins.prefixes);

// /**************** LIFTOFF IN 3... 2... 1... ****************/
styleSheet.inject(); //eslint-disable-line indent
// /****************      TO THE MOOOOOOON     ****************/

// define some constants 
var isBrowser = typeof document !== 'undefined';
var isDev = function (x) {
  return x === 'development' || !x;
}(process.env.NODE_ENV);
var isTest = process.env.NODE_ENV === 'test';

// import some helpers 
// hashes a string to something 'unique'

// takes a string, converts to lowercase, strips out nonalphanumeric.
function simple(str) {
  return str.toLowerCase().replace(/[^a-z0-9]/g, '');
}

/**** simulations  ****/

// a flag to enable simulation meta tags on dom nodes 
// defaults to true in dev mode. recommend *not* to 
// toggle often. 
var canSimulate = isDev;

// we use these flags for issuing warnings when simulate is called 
// in prod / in incorrect order 
var warned1 = false,
    warned2 = false;

// toggles simulation activity. shouldn't be needed in most cases 
function simulations() {
  var bool = arguments.length <= 0 || arguments[0] === undefined ? true : arguments[0];

  canSimulate = !!bool;
}

// use this on dom nodes to 'simulate' pseudoclasses
// <div {...hover({ color: 'red' })} {...simulate('hover', 'visited')}>...</div>
// you can even send in some weird ones, as long as it's in simple format 
// and matches an existing rule on the element 
// eg simulate('nthChild2', ':hover:active') etc 
function simulate() {
  if (!canSimulate) {
    if (!warned1) {
      console.warn('can\'t simulate without once calling simulations(true)'); //eslint-disable-line no-console
      warned1 = true;
    }
    if (!isDev && !isTest && !warned2) {
      console.warn('don\'t use simulation outside dev'); //eslint-disable-line no-console
      warned2 = true;
    }
    return {};
  }

  for (var _len = arguments.length, pseudos = Array(_len), _key = 0; _key < _len; _key++) {
    pseudos[_key] = arguments[_key];
  }

  return pseudos.reduce(function (o, p) {
    return o['data-simulate-' + simple(p)] = '', o;
  }, {});
}

/**** labels ****/
// toggle for debug labels. 
// *shouldn't* have to mess with this manually
var hasLabels = isDev;

function cssLabels(bool) {
  hasLabels = !!bool;
}

// // clears out the cache and empties the stylesheet
// best for tests, though there might be some value for SSR. 
function flush() {
  // todo - tests 
  styleSheet.inserted = {};
  styleSheet.cache = {};
  styleSheet.flush();
  styleSheet.inject();
}

// escape hatchhhhhhh
// export function insertRule(css) {
//   styleSheet.insert(css)  
// }

function insertRule(css, id) {
  var hasId = !!id;
  id = id || (0, _hash2.default)(css).toString(36);
  if (!styleSheet.inserted[id]) {
    styleSheet.insert(css + (!hasId ? '/* $id: ' + id + ' */' : ''));
    styleSheet.inserted[id] = true;
  }
}

// now, some functions to help deal with styles / rules 

// converts a js style object to css markup
// for the umd build, we'll used browserify to extract react's 
// CSSPropertyOperations module and it's deps into ./CSSPropertyOperations 

// generates a hash for (type, style)
function styleHash(type, style) {
  // todo - default type = '_'. this changes all the hashes and will break tests, so do later 
  // make sure type exists
  // make sure obj is style-like?
  return (0, _hash2.default)(type + Object.keys(style).reduce(function (str, k) {
    return str + k + style[k];
  }, '')).toString(36);
}

// generates a css selector for (id, type)
function selector(id, type) {
  // id should exist
  var isFullSelector = type && type[0] === '$';
  var isParentSelector = type && type[0] === '%';
  var cssType = type === '_' ? '' : // plain style object
  isFullSelector ? type.slice(1) : // via select()
  isParentSelector ? type.slice(1) : // via parent()
  ':' + type; // pseudo
  var result = void 0;

  if (isFullSelector) {
    result = cssType.split(',').map(function (x) {
      return '[data-css-' + id + ']' + x;
    }).join(',');
  } else if (isParentSelector) {
    // todo - attach pseudo classes if any
    result = cssType.split(',').map(function (x) {
      return x + ' [data-css-' + id + ']';
    }).join(',');
  } else {
    result = '[data-css-' + id + ']' + cssType;
  }

  // https://github.com/threepointone/glamor/issues/20
  // todo - only on chrome versions and server side 
  result = result.replace(/\:hover/g, ':hover:nth-child(n)');

  if (canSimulate && type !== '_' && !isFullSelector && !isParentSelector && cssType[0] === ':') {
    // todo - work with pseudo selector on full selector at least 
    result += ', [data-css-' + id + '][data-simulate-' + simple(type) + ']';
  }
  return result;
}

// ... which is then used to generate css rules 

function cssrule(id, type, style) {
  var result = plugins.apply({ id: id, type: type, style: style, selector: selector(id, type) });

  return result.selector + '{ ' + (0, _CSSPropertyOperations.createMarkupForStyles)(result.style) + ' } ';
}

// given a rule {data-css-id: ''}, checks if it's a valid, registered id
// returns the id 
function idFor(rule) {
  // todo - weak map hash for this?
  if (Object.keys(rule).length !== 1) throw new Error('not a rule');
  var regex = /data\-css\-([a-zA-Z0-9]+)/;
  var match = regex.exec(Object.keys(rule)[0]);
  if (!match) throw new Error('not a rule');
  return match[1];
}

// checks if a rule is registered
function isRule(rule) {
  try {
    var id = idFor(rule);
    return id && styleSheet.cache[id];
  } catch (e) {
    return false;
  }
}

// a generic rule creator/insertor 
function add() {
  var type = arguments.length <= 0 || arguments[0] === undefined ? '_' : arguments[0];
  var style = arguments[1];

  var id = styleHash(type, style),
      // generate a hash based on type/style, use this to 'id' the rule everywhere 
  label = '';
  if (!styleSheet.cache[id]) {
    insertRule(cssrule(id, type, style), id);
    styleSheet.cache[id] = { type: type, style: style, id: id };
  }
  if (hasLabels) {
    // adds a debug label 
    label = style.label || label;
  }

  return _defineProperty({}, 'data-css-' + id, label);
}

// with those in place, we can now define user-friendly functions for 
// defining styles on nodes 

// first up, what will probably be most commonly used.
// defines some css 'directly' on the node it's applied on
function style(obj) {
  return add(undefined, obj);
}

// alllllll the pseudoclasses
// todo - autogenerate this by scraping MDN
function active(x) {
  return add('active', x);
}

function any(x) {
  return add('any', x);
}

function checked(x) {
  return add('checked', x);
}

function disabled(x) {
  return add('disabled', x);
}

function empty(x) {
  return add('empty', x);
}

function enabled(x) {
  return add('enabled', x);
}

function _default(x) {
  return add('default', x); // note '_default' name  
}

function first(x) {
  return add('first', x);
}

function firstChild(x) {
  return add('first-child', x);
}

function firstOfType(x) {
  return add('first-of-type', x);
}

function fullscreen(x) {
  return add('fullscreen', x);
}

function focus(x) {
  return add('focus', x);
}

function hover(x) {
  return add('hover', x);
}

function indeterminate(x) {
  return add('indeterminate', x);
}

function inRange(x) {
  return add('in-range', x);
}

function invalid(x) {
  return add('invalid', x);
}

function lastChild(x) {
  return add('last-child', x);
}

function lastOfType(x) {
  return add('last-of-type', x);
}

function left(x) {
  return add('left', x);
}

function link(x) {
  return add('link', x);
}

function onlyChild(x) {
  return add('only-child', x);
}

function onlyOfType(x) {
  return add('only-of-type', x);
}

function optional(x) {
  return add('optional', x);
}

function outOfRange(x) {
  return add('out-of-range', x);
}

function readOnly(x) {
  return add('read-only', x);
}

function readWrite(x) {
  return add('read-write', x);
}

function required(x) {
  return add('required', x);
}

function right(x) {
  return add('right', x);
}

function root(x) {
  return add('root', x);
}

function scope(x) {
  return add('scope', x);
}

function target(x) {
  return add('target', x);
}

function valid(x) {
  return add('valid', x);
}

function visited(x) {
  return add('visited', x);
}

// parameterized pseudoclasses
function dir(p, x) {
  return add('dir(' + p + ')', x);
}
function lang(p, x) {
  return add('lang(' + p + ')', x);
}
function not(p, x) {
  // should this be a plugin?
  var selector = p.split(',').map(function (x) {
    return x.trim();
  }).map(function (x) {
    return ':not(' + x + ')';
  });
  if (selector.length === 1) {
    return add('not(' + p + ')', x);
  }
  return select(selector.join(''), x);
}
function nthChild(p, x) {
  return add('nth-child(' + p + ')', x);
}
function nthLastChild(p, x) {
  return add('nth-last-child(' + p + ')', x);
}
function nthLastOfType(p, x) {
  return add('nth-last-of-type(' + p + ')', x);
}
function nthOfType(p, x) {
  return add('nth-of-type(' + p + ')', x);
}

// pseudoelements
function after(x) {
  return add(':after', x);
}
function before(x) {
  return add(':before', x);
}
function firstLetter(x) {
  return add(':first-letter', x);
}
function firstLine(x) {
  return add(':first-line', x);
}
function selection(x) {
  return add(':selection', x);
}
function backdrop(x) {
  return add(':backdrop', x);
}
function placeholder(x) {
  // https://github.com/threepointone/glamor/issues/14
  return merge(add(':placeholder', x), add(':-webkit-input-placeholder', x), add(':-moz-placeholder', x), add(':-ms-input-placeholder', x));
}

// unique feature 
// when you need to define 'real' css (whatever that may be)
// https://twitter.com/threepointone/status/756585907877273600
// https://twitter.com/threepointone/status/756986938033254400
function select(selector, _style) {
  if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object') {
    return style(selector);
  }
  //  todo - warn when missing possible space
  return add('$' + selector, _style); // signalling ahead that this is a plain selector 
}

// alias. bringin' back jquery
var $ = exports.$ = select;

function parent(selector, style) {
  return add('%' + selector, style);
}

// we define a function to 'merge' styles together.
// backstory - because of a browser quirk, multiple styles are applied in the order they're 
// defined the stylesheet, not in the order of application 
// in most cases, this won't case an issue UNTIL IT DOES 
// instead, use merge() to merge styles,
// with latter styles gaining precedence over former ones 

// todo - this needs a refactor urghhh
function merge() {
  var labels = [],
      mergeLabel = void 0,
      styleBag = {},
      mediaBag = {};

  for (var _len2 = arguments.length, rules = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    rules[_key2] = arguments[_key2];
  }

  rules.forEach(function (rule, i) {
    // optionally send a string as first argumnet to 'label' this merged rule  
    if (i === 0 && typeof rule === 'string') {
      mergeLabel = rule;
      // bail early
      return;
    }
    if (isRule(rule)) {
      // it's a rule!

      var _id = idFor(rule);

      if (styleSheet.cache[_id].bag) {
        var _ret = function () {
          // merged rule 

          var _styleSheet$cache$_id = styleSheet.cache[_id];
          var bag = _styleSheet$cache$_id.bag;
          var label = _styleSheet$cache$_id.label;
          var media = _styleSheet$cache$_id.media;

          Object.keys(bag).forEach(function (type) {
            styleBag[type] = _extends({}, styleBag[type] || {}, bag[type]);
          });
          // if there's a media bag, merge those in 
          if (media) {
            Object.keys(media).forEach(function (expr) {
              mediaBag[expr] = mediaBag[expr] || {};
              Object.keys(media[expr]).forEach(function (type) {
                // mediaBag[expr][type] = mediaBag[expr][type] || {}
                mediaBag[expr][type] = _extends({}, mediaBag[expr][type] || {}, media[expr][type]);
              });
            });
          }

          hasLabels && labels.push('[' + (label || '*') + ']');
          return {
            v: void 0
          };
          // that was fairly straightforward
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      }

      if (styleSheet.cache[_id].expr) {
        var _ret2 = function () {
          // media rule
          var _styleSheet$cache$_id2 = styleSheet.cache[_id];
          var expr = _styleSheet$cache$_id2.expr;
          var label = _styleSheet$cache$_id2.label;
          var rule = _styleSheet$cache$_id2.rule;
          var style = _styleSheet$cache$_id2.style;

          mediaBag[expr] = mediaBag[expr] || {};
          if (rule) {
            var iid = idFor(rule);
            if (styleSheet.cache[iid].bag) {
              (function () {
                // if merged rule, merge it's bag into stylebag 
                // we won't expect a mediabag in this merged rule, because it would have thrown in media (phew)

                var bag = styleSheet.cache[iid].bag;

                Object.keys(bag).forEach(function (type) {
                  mediaBag[expr][type] = _extends({}, mediaBag[expr][type] || {}, bag[type]);
                });
              })();
            } else {
              var _styleSheet$cache$iid = styleSheet.cache[iid];
              var type = _styleSheet$cache$iid.type;
              var _style2 = _styleSheet$cache$iid.style;

              mediaBag[expr][type] = _extends({}, mediaBag[expr][type] || {}, _style2);
            }
          } else {
            mediaBag[expr]._ = _extends({}, mediaBag[expr]._ || {}, style);
          }

          // mediaBag[expr].push(rule)
          hasLabels && labels.push(label);
          return {
            v: void 0
          };

          // throw new Error('cannot merge a media rule')
        }();

        if ((typeof _ret2 === 'undefined' ? 'undefined' : _typeof(_ret2)) === "object") return _ret2.v;
      } else {
        // simple rule 

        var _styleSheet$cache$_id3 = styleSheet.cache[_id];
        var type = _styleSheet$cache$_id3.type;
        var _style3 = _styleSheet$cache$_id3.style;

        styleBag[type] = _extends({}, styleBag[type] || {}, _style3);
        hasLabels && labels.push(_style3.label || '*'); // todo - match 'add()'s original label
        return;
        // not too bad 
      }
    } else {
      // plain style 
      styleBag._ = _extends({}, styleBag._ || {}, rule);
      hasLabels && labels.push('{:}');
    }
  });

  // todo - remove label from merged styles? unclear. 

  var id = (0, _hash2.default)(mergeLabel + JSON.stringify(mediaBag) + JSON.stringify(styleBag)).toString(36); // todo - predictable order
  // make a merged label
  var label = hasLabels ? '' + (mergeLabel ? mergeLabel + '= ' : '') + (labels.length ? labels.join(' + ') : '') : ''; // yuck 

  if (!styleSheet.cache[id]) {
    if (!styleSheet.inserted[id]) {
      Object.keys(styleBag).forEach(function (type) {
        styleSheet.insert(cssrule(id, type, styleBag[type]));
      });

      Object.keys(mediaBag).forEach(function (expr) {
        var css = Object.keys(mediaBag[expr]).map(function (type) {
          return cssrule(id, type, mediaBag[expr][type]);
        });
        var result = plugins.media.apply({ id: id, expr: expr, css: css });
        styleSheet.insert('@media ' + result.expr + ' { ' + result.css.join('\n') + ' }');
      });
      styleSheet.inserted[id] = true;
    }

    styleSheet.cache[id] = _extends({ bag: styleBag, id: id, label: label }, Object.keys(mediaBag).length > 0 ? { media: mediaBag } : {});
  }
  return _defineProperty({}, 'data-css-' + id, label);
}

var compose = exports.compose = merge;

function isMerged(id) {
  return !!styleSheet.cache[id].bag;
}

function isMedia(id) {
  return !!styleSheet.cache[id].expr;
}

function insertMediaRule(_ref3) {
  var id = _ref3.id;
  var expr = _ref3.expr;
  var style = _ref3.style;
  var rule = _ref3.rule;
  var label = _ref3.label;
  var css = _ref3.css;

  if (!styleSheet.inserted[id]) {
    var result = plugins.media.apply({ id: id, expr: expr, css: css });
    styleSheet.insert('@media ' + result.expr + ' { ' + result.css.join('\n') + ' }');
    styleSheet.inserted[id] = true;
  }
  styleSheet.cache[id] = { expr: expr, rule: rule, style: style, id: id, label: label };
}

// this one's for media queries 
// they cannot be merged with other queries 
// todo - we should test whether the query is valid and give dev feedback 
function media(expr) {
  for (var _len3 = arguments.length, rules = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    rules[_key3 - 1] = arguments[_key3];
  }

  if (rules.length > 1) {
    return media(expr, merge.apply(undefined, rules));
  } // todo - iterate yourself instead 
  var rule = rules[0];
  // test if valid media query
  if (isRule(rule)) {
    var id = idFor(rule);
    var spec = styleSheet.cache[id];
    if (isMerged(id)) {
      var _ret4 = function () {
        // merged rule        
        if (spec.media) {
          throw new Error('cannot apply a media rule onto another');
        }
        var bag = spec.bag;

        var newId = (0, _hash2.default)(expr + id).toString(36);
        var label = hasLabels ? '*mq [' + spec.label + ']' : '';

        if (!styleSheet.cache[newId]) {
          var css = Object.keys(bag).map(function (type) {
            return cssrule(newId, type, bag[type]);
          });
          insertMediaRule({ id: newId, expr: expr, rule: rule, label: label, css: css });
        }

        return {
          v: _defineProperty({}, 'data-css-' + newId, label)
        };
        // easy 
      }();

      if ((typeof _ret4 === 'undefined' ? 'undefined' : _typeof(_ret4)) === "object") return _ret4.v;
    } else if (isMedia(id)) {
      // media rule
      throw new Error('cannot apply a media rule onto another');
    } else {
      // simple rule
      var _newId = (0, _hash2.default)(expr + id).toString(36);
      var _label = hasLabels ? '*mq ' + (spec.style.label || '*') : '';

      if (!styleSheet.cache[_newId]) {
        var css = [cssrule(_newId, spec.type, spec.style)];
        insertMediaRule({ id: _newId, expr: expr, rule: rule, label: _label, css: css });
      }

      return _defineProperty({}, 'data-css-' + _newId, _label);
      // easier 
    }
  } else {
    // a plain style 
    var _style4 = rule;
    var _newId2 = styleHash(expr, _style4);
    var _label2 = hasLabels ? '*mq ' + (_style4.label || '*') : '';
    if (!styleSheet.cache[_newId2]) {
      var _css = [cssrule(_newId2, '_', _style4)];
      insertMediaRule({ id: _newId2, expr: expr, style: rule, label: _label2, css: _css });
    }
    return _defineProperty({}, 'data-css-' + _newId2, _label2);
  }
}

var presets = exports.presets = {
  mobile: '(min-width: 400px)',
  phablet: '(min-width: 550px)',
  tablet: '(min-width: 750px)',
  desktop: '(min-width: 1000px)',
  hd: '(min-width: 1200px)'
};

/**** live media query labels ****/

// simplest implementation -
// cycle through the cache, and for every media query
// find matching elements and update the label 
function updateMediaQueryLabels() {
  Object.keys(styleSheet.cache).forEach(function (id) {
    var expr = styleSheet.cache[id].expr;

    if (expr && hasLabels && window.matchMedia) {
      (function () {
        var els = document.querySelectorAll('[data-css-' + id + ']');
        var match = window.matchMedia(expr).matches ? '✓' : '✕';
        var regex = /^(✓|✕|\*)mq/;
        [].concat(_toConsumableArray(els)).forEach(function (el) {
          return el.setAttribute('data-css-' + id, el.getAttribute('data-css-' + id).replace(regex, match + 'mq'));
        });
      })();
    }
  });
}

// saves a reference to the loop we trigger 
var interval = void 0;

function trackMediaQueryLabels() {
  var bool = arguments.length <= 0 || arguments[0] === undefined ? true : arguments[0];
  var period = arguments.length <= 1 || arguments[1] === undefined ? 2000 : arguments[1];

  if (bool) {
    if (interval) {
      console.warn('already tracking labels, call trackMediaQueryLabels(false) to stop'); // eslint-disable-line no-console 
      return;
    }
    interval = setInterval(function () {
      return updateMediaQueryLabels();
    }, period);
  } else {
    clearInterval(interval);
    interval = null;
  }
}

// in dev mode, start this up immediately 
if (isDev && isBrowser) {
  trackMediaQueryLabels(true);
  // todo - make sure hot loading isn't broken
  // todo - clearInterval on browser close  
}

// we don't go all out for fonts as much, giving a simple font loading strategy 
// use a fancier lib if you need moar power
function fontFace(font) {
  var id = (0, _hash2.default)(JSON.stringify(font)).toString(36);
  if (!styleSheet.cache[id]) {
    styleSheet.cache[id] = { id: id, family: font.fontFamily, font: font };
    // todo - crossbrowser 
    insertRule('@font-face { ' + (0, _CSSPropertyOperations.createMarkupForStyles)(font) + '}', id);
    // if(!styleSheet.inserted[id]) {
    //   styleSheet.insert()  
    //   styleSheet.inserted[id] = true
    // }
  }
  return font.fontFamily;
}

// we can add keyframes in a similar manner, but still generating a unique name 
// for including in styles. this gives us modularity, but still a natural api 
function keyframes(name, kfs) {
  if (typeof name !== 'string') {
    kfs = name;
    name = 'animate';
  }
  var id = (0, _hash2.default)(name + JSON.stringify(kfs)).toString(36);
  if (!styleSheet.cache[id]) {
    styleSheet.cache[id] = { id: id, name: name, kfs: kfs };

    if (!styleSheet.inserted[id]) {
      (function () {
        styleSheet.inserted[id] = true;
        var inner = Object.keys(kfs).map(function (kf) {
          var result = plugins.keyframes.apply({ id: id, name: kf, style: kfs[kf] });
          return result.name + ' { ' + (0, _CSSPropertyOperations.createMarkupForStyles)(result.style) + '}';
        }).join('\n');

        ['-webkit-', '-moz-', '-o-', ''].forEach(function (prefix) {
          return styleSheet.insert('@' + prefix + 'keyframes ' + (name + '_' + id) + ' { ' + inner + '}');
        });
      })();
    }
  }
  return name + '_' + id;
}

/*** helpers for web components ***/
// https://github.com/threepointone/glamor/issues/16

function cssFor() {
  for (var _len4 = arguments.length, rules = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    rules[_key4] = arguments[_key4];
  }

  var ids = rules.reduce(function (o, r) {
    return o[idFor(r)] = true, o;
  }, {});
  var css = styleSheet.rules().map(function (_ref6) {
    var cssText = _ref6.cssText;

    var regex = /\[data\-css\-([a-zA-Z0-9]+)\]/gm;
    var match = regex.exec(cssText);

    if (match && ids[match[1]]) {
      return cssText;
    }
  }).filter(function (x) {
    return !!x;
  }).join('\n');
  return css;
}

function attribsFor() {
  for (var _len5 = arguments.length, rules = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
    rules[_key5] = arguments[_key5];
  }

  var htmlAttributes = rules.map(function (rule) {
    idFor(rule); // throwaway check for rule 
    var key = Object.keys(rule)[0],
        value = rule[key];
    return key + '="' + (value || '') + '"';
  }).join(' ');

  return htmlAttributes;
}

// a useful utility for quickly tapping objects. use with the :: operator 
// {x: 1}::log()
// [5, 12, 90]::log().filter(x => x%5)::log()
function log(msg) {
  //eslint-disable-line no-unused-vars
  console.log(msg || this); //eslint-disable-line no-console
  return this;
}