'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* 

high performance StyleSheet for css-in-js systems 

- uses multiple style tags behind the scenes for millions of rules 
- uses `insertRule` for appending in production for *much* faster performance
- 'polyfills' on server side 


// usage

import StyleSheet from 'glamor/lib/sheet'
let styleSheet = new StyleSheet({ 
  name: 'mySheet', // optional
  length: 30000 // defaults to 4000 for IE9, 65000 otherwise 
})

styleSheet.inject() 

// 'injects' the stylesheet into the page (or into memory if on server)

styleSheet.insert('#box { border: 1px solid red; }') 

// appends a css rule into the stylesheet 

styleSheet.rules()

// array of css rules 
// use for server side rendering, etc 

styleSheet.flush() 

// empties the stylesheet of all its contents


*/

function last() {
  return this[this.length - 1];
}

function sheetForTag(tag) {
  for (var i = 0; i < document.styleSheets.length; i++) {
    if (document.styleSheets[i].ownerNode === tag) {
      return document.styleSheets[i];
    }
  }
}

var isBrowser = typeof document !== 'undefined';
var isDev = function (x) {
  return x === 'development' || !x;
}(process.env.NODE_ENV);
var isTest = process.env.NODE_ENV === 'test';

var oldIE = function () {
  if (isBrowser) {
    var div = document.createElement('div');
    div.innerHTML = '<!--[if lt IE 10]><i></i><![endif]-->';
    return div.getElementsByTagName('i').length === 1;
  }
}();

function makeStyleTag() {
  var name = arguments.length <= 0 || arguments[0] === undefined ? '_css_' : arguments[0];

  var tag = document.createElement('style');
  tag.type = 'text/css';
  tag.id = name;
  tag.setAttribute('id', name);
  tag.appendChild(document.createTextNode(''));
  (document.head || document.getElementsByTagName('head')[0]).appendChild(tag);
  return tag;
}

var sheetCounter = 0;

var StyleSheet = exports.StyleSheet = function () {
  function StyleSheet(_ref) {
    var _ref$name = _ref.name;
    var name = _ref$name === undefined ? '_css_' + sheetCounter++ : _ref$name;
    var _ref$speedy = _ref.speedy;
    var speedy = _ref$speedy === undefined ? !isDev && !isTest : _ref$speedy;
    var _ref$length = _ref.length;
    var length = _ref$length === undefined ? isBrowser && oldIE ? 4000 : 65000 : _ref$length;

    _classCallCheck(this, StyleSheet);

    this.name = name;
    this.speedy = speedy; // the big drawback here is that the css won't be editable in devtools
    this.sheet = undefined;
    this.tags = [];
    this.length = length;
    this.ctr = 0;
  }

  _createClass(StyleSheet, [{
    key: 'inject',
    value: function inject() {
      var _this = this;

      if (this.injected) {
        throw new Error('already injected stylesheet!');
      }
      if (isBrowser) {
        // this section is just weird alchemy I found online off many sources 
        this.tags[0] = makeStyleTag(this.name + this.ctr);
        // this weirdness brought to you by firefox 
        this.sheet = sheetForTag(this.tags[0]);
      } else {
        // server side 'polyfill'. just enough behavior to be useful.
        this.sheet = {
          cssRules: [],
          insertRule: function insertRule(rule) {
            // enough 'spec compliance' to be able to extract the rules later  
            // in other words, just the cssText field 
            _this.sheet.cssRules.push({ cssText: rule });
          }
        };
      }
      this.injected = true;
    }
  }, {
    key: '_insert',
    value: function _insert(rule) {
      // this weirdness for perf, and chrome's weird bug 
      // https://stackoverflow.com/questions/20007992/chrome-suddenly-stopped-accepting-insertrule
      try {
        this.sheet.insertRule(rule, this.sheet.cssRules.length); // todo - correct index here     
      } catch (e) {
        if (isDev) {
          // might need beter dx for this 
          console.warn('whoops, illegal rule inserted', rule); //eslint-disable-line no-console
        }
      }
    }
  }, {
    key: 'insert',
    value: function insert(rule) {
      var _context;

      // more browser weirdness. I don't even know
      if (this.tags.length > 0 && (_context = this.tags, last).call(_context).styleSheet) {
        var _context2;

        (_context2 = this.tags, last).call(_context2).styleSheet.cssText += rule;
      } else {
        if (isBrowser) {
          // here goes the logic for 
          if (this.speedy && this.sheet.insertRule) {
            this._insert(rule);
          } else {
            var _context3;

            (_context3 = this.tags, last).call(_context3).appendChild(document.createTextNode(rule));

            // todo - more efficent here please 
            if (!this.speedy) {
              var _context4;

              // sighhh
              this.sheet = sheetForTag((_context4 = this.tags, last).call(_context4));
            }
          }
        } else {
          // server side is pretty simple         
          this.sheet.insertRule(rule);
        }
      }
      this.ctr++;
      if (isBrowser && this.ctr % this.length === 0) {
        var _context5;

        this.tags.push(makeStyleTag(this.name + Math.round(this.ctr / this.length)));
        this.sheet = sheetForTag((_context5 = this.tags, last).call(_context5));
      }
    }
  }, {
    key: 'flush',
    value: function flush() {
      // todo backward compat (styleTag.styleSheet.cssText?)
      if (isBrowser) {
        this.tags.forEach(function (tag) {
          return tag.parentNode.removeChild(tag);
        });
        this.tags = [];
        this.sheet = null;
        // todo - look for remnants in document.styleSheets
      } else {
        // simpler on server 
        this.sheet.cssRules = [];
      }
      this.injected = false;
    }
  }, {
    key: 'rules',
    value: function rules() {
      if (!isBrowser) {
        return Array.from(this.sheet.cssRules);
      }
      return this.tags.reduce(function (arr, tag) {
        return arr.concat(Array.from(sheetForTag(tag).cssRules));
      }, []);
    }
  }]);

  return StyleSheet;
}();